// ---- file   : tksmysql.h
// ---- author : Bastian Spiegel <bs@tkscript.de> <fli@tkscript.de>
// ---- license: GPL
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	*/
// ---- created: 29012002 <fli> 
// ---- changed: 16062003 <fli> now it's a plugin

#define YAC_PRINTF
#define YAC_BIGSTRING

//#define MYSQL_HAVE_CHANGEUSER 


#include <stdio.h>
#include <stdarg.h>
#include <math.h>

#include <yac.h>
#include <yac_host.cpp>
YAC_Host *yac_host;

#ifdef YAC_WIN32
#include <windows.h>
#endif
#include <mysql.h>
#include "tksmysql.h"


YG("mysql");

// --------------------------------------------------------------------------- MySQLConnection
YC class _MySQLConnection : public TKSMySQLConnection {
    YAC(_MySQLConnection);

    YM //sSI  _connect       (YAC_Object *_dbname, YAC_Object *_hostname, YAC_Object *_username, YAC_Object *_passwd);
    YM //sSI  use            (YAC_Object *_dbname);
    YM //void _close         (void);
    YM //sSI  query          (YAC_Object *_querystring);
    YM //sSI  affectedRows   (void);
    YM //void setPort        (sSI);
    YM //sSI  changeUser     (YAC_Object *_uname, YAC_Object *_passwd);
    YM //sSI  createDB       (YAC_Object *_dbname);
    YM //void getErrorString (YAC_Value *_r);

};
#include "ying_mysql_MySQLConnection.cpp"

// ---------------------------------------------------------------------------
YC class _MySQLRow : public TKSMySQLRow {
    YAC(_MySQLRow);

    YM //void getFieldByName (YAC_Object *_fieldname, YAC_Value *_r);
    YM //void getFieldByIndex(sSI _id, YAC_Value *_r);
};
#include "ying_mysql_MySQLRow.cpp"

#include "ying_mysql.cpp"


// ---------------------------------------------------------------------------
void YAC_Init(YAC_Host *_host) {
	yac_host=_host;
    if(yac_host->yacGetDebugLevel())
        yac_host->yacPrint("mysql::YAC_Init() called.");

    YAC_Init_mysql(_host);

    if(yac_host->yacGetDebugLevel())
        yac_host->yacPrint("mysql::YAC_Init() finished.");
}

void YAC_Exit(YAC_Host *_host) {
    if(yac_host->yacGetDebugLevel())
        yac_host->yacPrint("mysql::YAC_Exit() called.");

    YAC_Exit_mysql(_host);

    if(yac_host->yacGetDebugLevel())
        yac_host->yacPrint("mysql::YAC_Init() finished.");
}

sUI YAC_Version(void) {
	return 0x00020000;
}





//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//~                                                                ~
//~ TKSMySQLConnection                                             ~
//~                                                                ~
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
TKSMySQLConnection::TKSMySQLConnection(void) {
    portnr=MYSQL_PORT; //3306;
    connected=0;
    mysql_init(&sql_handler);
    last_result=0;
}

TKSMySQLConnection::~TKSMySQLConnection() {
    _close();
}

YAC_Iterator *TKSMySQLConnection::yacGetIterator(void) {
    return new MySQLRowIterator(this);
}


sSI TKSMySQLConnection::_connect(YAC_Object *_dbname, YAC_Object *_hostname, YAC_Object *_username, YAC_Object *_passwd) {
    const char *cs_hostname=0;
    const char *cs_dbname=0;
    const char *cs_username=0;
    const char *cs_passwd=0;

    if(YAC_CHK(_dbname, YAC_CLID_STRING))
    {
        YAC_String *s_dbname=(YAC_String*)_dbname;
        cs_dbname=(const char*)s_dbname->chars;
    }
    if(YAC_CHK(_hostname, YAC_CLID_STRING))
    {
        YAC_String *s_hostname=(YAC_String*)_hostname;
        cs_hostname=(const char*)s_hostname->chars;
    }
    if(YAC_CHK(_username, YAC_CLID_STRING))
    {
        YAC_String *s_username=(YAC_String*)_username;
        cs_username=(const char*)s_username->chars;
    }
    if(YAC_CHK(_passwd, YAC_CLID_STRING))
    {
        YAC_String *s_passwd=(YAC_String*)_passwd;
        cs_passwd=(const char*)s_passwd->chars;
    }
    if(mysql_real_connect(&sql_handler, cs_hostname, cs_username, cs_passwd, cs_dbname, portnr, 0 /* unix_socket */, 0 /* client flag */))
    {
        connected=1;
    }
    else
    {
		::fprintf(stderr, "[---] MySQL connection to host \"%s\" port %i (user=\"%s\", database=\"%s\") failed.\n",
           cs_hostname, portnr, cs_username, cs_dbname);
    }
    return (connected);
}

sSI TKSMySQLConnection::use(YAC_Object *_dbname) {
    if(connected)
    {
        if(YAC_CHK(_dbname, YAC_CLID_STRING))
        {
            YAC_String *dbname=(YAC_String*)_dbname;
            const char *cs_dbname=(const char*)dbname->chars;
            if(!mysql_select_db(&sql_handler, cs_dbname))
            {
                /** database changed **/
                return 1;
            }
        }
    }
    return 0;
}

void TKSMySQLConnection::freeResult(void) {
    if(last_result)
    {
        mysql_free_result(last_result);
        last_result=0;
    }
}

void TKSMySQLConnection::_close(void) {
    if(connected)
    {
        freeResult();
        mysql_close(&sql_handler);
        connected=0;
    }
}

sSI TKSMySQLConnection::query(YAC_Object *_querystring) {
    if(connected)
    {
        if(YAC_CHK(_querystring, YAC_CLID_STRING))
        {
            YAC_String *querystring=(YAC_String*)_querystring;
            const char *cs_query=(const char*)querystring->chars;
            if(!mysql_query(&sql_handler, cs_query))
            {
                /** query succeeded **/
                last_result=mysql_store_result(&sql_handler);
                return 1;
            }
        }
    }
    return 0;
}

sSI TKSMySQLConnection::affectedRows(void) {
    if(connected)
    {
        int r=(int)mysql_affected_rows(&sql_handler);
        if(r<0)
            r=0;
        return r;
    }
    else
        return -1;
}

void TKSMySQLConnection::setPort(sSI _portnr) {
    if(_portnr>0)
        portnr=_portnr;
}

sSI TKSMySQLConnection::changeUser(YAC_Object *_uname, YAC_Object *_passwd) {
#ifdef MYSQL_HAVE_CHANGEUSER 
    if(connected)
    {
        const char *cs_uname=0;
        const char *cs_passwd=0;
        if(YAC_CHK(_uname, YAC_CLID_STRING))
        {
            YAC_String *uname=(YAC_String*)_uname;
            cs_uname=(const char*)uname->chars;
        }
        if(YAC_CHK(_passwd, YAC_CLID_STRING))
        {
            YAC_String *s_passwd=(YAC_String*)_passwd;
            cs_passwd=(const char*)s_passwd->chars;
        }
        if(!mysql_change_user(&sql_handler, cs_uname, cs_passwd, 0 /* dbname */))
            return 1;
    } 
#endif
    return 0;
}


sSI TKSMySQLConnection::createDB(YAC_Object *_dbname) {
    if(connected)
    {
        if(YAC_CHK(_dbname, YAC_CLID_STRING))
        {
            YAC_String *dbname=(YAC_String*)_dbname;
            const char *cs_dbname=(const char*)dbname->chars;
            if(!mysql_create_db(&sql_handler, cs_dbname))
                return 1;
        } 
    }
    return 0;
}

void TKSMySQLConnection::getErrorString(YAC_Value *_r) {
    char *es=mysql_error(&sql_handler);
    if(es)
    {
        YAC_String *ns=(YAC_String*)yac_host->yacNewByID(YAC_CLID_STRING);
        ns->visit(es);
        YAC_RETS(ns, 1);
        return;
    }
    YAC_RETS(0, 0);
}



//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//~                                                                ~
//~ TKSMySQLRow                                                    ~
//~                                                                ~
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
TKSMySQLRow::TKSMySQLRow(void) {
    con=0;
    last_row=0;
    fields=0;
    num_fields=0;
}

TKSMySQLRow::~TKSMySQLRow() {}

void TKSMySQLRow::fetchFields(void) {
    if(con->last_result)
    {
        fields=mysql_fetch_fields(con->last_result);
        num_fields=mysql_num_fields(con->last_result);
    }
    else
    {
        fields=0;
        num_fields=0;
    }
}

void TKSMySQLRow::fetchRow(void) {
    if(con->last_result)
        last_row=mysql_fetch_row(con->last_result);
}

void TKSMySQLRow::getFieldByName(YAC_Object *_fieldname, YAC_Value *_r) {
    if(YAC_CHK(_fieldname, YAC_CLID_STRING))
    {
        YAC_String *fieldname=(YAC_String*)_fieldname;
        /** find field index for _fieldname **/
        int i;
        for(i=0; i<num_fields; i++)
        {
            if(fieldname->compare(fields[i].name))
            {
                getFieldByIndex(i, _r);
                return;
            }
        }
    }
    YAC_RETI(0);
}

void TKSMySQLRow::getFieldByIndex(sSI _i, YAC_Value *_r) {
    if(last_row && _i>=0 && _i<num_fields)
    {
        //clogf("fields[%i] value=\"%s\".\n", _i, last_row[_i]);
        switch(fields[_i].type)
        {
        case FIELD_TYPE_CHAR:
        //case FIELD_TYPE_TINY:
            {
                //clogf("FIELD_TYPE_CHAR size=%i\n", fields[_i].length);
                int ival;
                ::sscanf(last_row[_i], "%d", &ival);
                YAC_RETI(ival);
            }
            break;
        case FIELD_TYPE_SHORT:
            {
                //clogf("FIELD_TYPE_SHORT size=%i\n", fields[_i].length);
                int ival;
                ::sscanf(last_row[_i], "%d", &ival);
                YAC_RETI(ival);
            }
            break;
        case FIELD_TYPE_LONG:
            {
                //clogf("FIELD_TYPE_LONG size=%i\n", fields[_i].length);
                int ival;
                ::sscanf(last_row[_i], "%d", &ival);
                YAC_RETI(ival);
            }
            break;
        case FIELD_TYPE_INT24:
            {
                //clogf("FIELD_TYPE_INT24 size=%i\n", fields[_i].length);
                int ival;
                ::sscanf(last_row[_i], "%d", &ival);
                YAC_RETI(ival);
            }
            break;
        case FIELD_TYPE_DECIMAL:
            {
                //clogf("FIELD_TYPE_DECIMAL size=%i\n", fields[_i].length);
                int ival;
                ::sscanf(last_row[_i], "%d", &ival);
                YAC_RETI(ival);
            }
            break;
        case FIELD_TYPE_FLOAT:
            {
                //clogf("FIELD_TYPE_FLOAT size=%i\n", fields[_i].length);
                float fval;
                ::sscanf(last_row[_i], "%g", &fval);
                YAC_RETF(fval);
            }
            break;
        case FIELD_TYPE_DOUBLE:
            {
                //clogf("FIELD_TYPE_DOUBLE size=%i\n", fields[_i].length);
                float fval;
                ::sscanf(last_row[_i], "%g", &fval);
                YAC_RETF(fval);
            }
            break;
         case FIELD_TYPE_VAR_STRING:
         case FIELD_TYPE_STRING:
            {
                //clogf("FIELD_TYPE_STRING size=%i\n", fields[_i].length);
                YAC_String *ns=(YAC_String*)yac_host->yacNew("String"); //new YAC_String();
                //ns->setEngine(tkscript);
                //ns->copy((char*)last_row[_i]);
				ns->visit((char*)last_row[_i]);
                YAC_RETS(ns, 1);
            }
            break;
        case FIELD_TYPE_TINY_BLOB:
        case FIELD_TYPE_MEDIUM_BLOB:
        case FIELD_TYPE_LONG_BLOB:
        case FIELD_TYPE_BLOB:
            {
                //clogf("FIELD_TYPE_BLOB size=%i\n", fields[_i].length);
                YAC_String *ns=(YAC_String*)yac_host->yacNew("String"); //new YAC_String();
                //ns->setEngine(tkscript);
                //ns->copy((char*)last_row[_i]);
				//ns->copy((char*)last_row[_i]);
				ns->visit((char*)last_row[_i]);
                YAC_RETS(ns, 1);
            }
            break;
        case FIELD_TYPE_LONGLONG:
        case FIELD_TYPE_TIMESTAMP:
        case FIELD_TYPE_DATE:
        case FIELD_TYPE_TIME:
        case FIELD_TYPE_DATETIME:
        case FIELD_TYPE_YEAR:
        case FIELD_TYPE_SET:
        case FIELD_TYPE_ENUM:
        case FIELD_TYPE_NULL:
        default:
            /** unsupported **/
            YAC_RETI(0);
            break;
        }
    }
}



//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//~                                                                ~
//~ MySQLRowIterator                                            ~
//~                                                                ~
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
MySQLRowIterator::MySQLRowIterator(TKSMySQLConnection *_con) {
    con=_con;
    row.con=_con;
}

void MySQLRowIterator::begin(void) {
    row.fetchFields();
}

void MySQLRowIterator::end(void) {
}

void MySQLRowIterator::getNext(YAC_Value *_r) {
    row.fetchRow();
    if(row.last_row) 
        _r->initObject(&row, 0);
    else
    {
        _r->value.int_val=0;
        _r->deleteme=0;
        _r->type=0xFF;
    }
}



