// ---- file   : tksmysql.h
// ---- author : Bastian Spiegel <bs@tkscript.de>
// ---- license: GPL
/*
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
	*/

#ifndef __TKS_MYSQL_H__
#define __TKS_MYSQL_H__

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//~                                                                ~
//~ TKSMySQLConnection                                             ~
//~                                                                ~
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class TKSMySQLConnection : public YAC_Object {
public:
    MYSQL      sql_handler;
    MYSQL_RES *last_result;
    int        portnr;      // defaults to 3306
    sBool      connected;   // 1 if connected to database

public:
    TKSMySQLConnection(void);
    ~TKSMySQLConnection();

    void freeResult(void);

    YAC_Iterator *yacGetIterator(void); /** get row iterator **/

    sSI  _connect       (YAC_Object *_dbname, YAC_Object *_hostname, YAC_Object *_username, YAC_Object *_passwd);
    sSI  use            (YAC_Object *_dbname);
    void _close         (void);
    sSI  query          (YAC_Object *_querystring);
    sSI  affectedRows   (void);
    void setPort        (sSI);
    sSI  changeUser     (YAC_Object *_uname, YAC_Object *_passwd);
    sSI  createDB       (YAC_Object *_dbname);
    void getErrorString (YAC_Value *_r);
};


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//~                                                                ~
//~ TKSMySQLRow                                                    ~
//~                                                                ~
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class TKSMySQLRow : public YAC_Object {
public:
    TKSMySQLConnection *con;
    MYSQL_ROW           last_row;
    MYSQL_FIELD        *fields;
    int                 num_fields;

public:
    TKSMySQLRow(void);
    ~TKSMySQLRow();

    void fetchFields(void);
    void fetchRow(void);

    void getFieldByName (YAC_Object *_fieldname, YAC_Value *_r);
    void getFieldByIndex(sSI _id, YAC_Value *_r);
};


//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//~                                                                ~
//~ TKSMySQLRowIterator                                            ~
//~                                                                ~
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
class MySQLRowIterator : public YAC_Iterator {
public:
	TKSMySQLConnection *con;
    TKSMySQLRow         row;

public:
	//MySQLRowIterator(void) { }
	//~MySQLRowIterator() { }

	MySQLRowIterator(TKSMySQLConnection *_con);
    
    void begin  (void);
	void getNext(YAC_Value *_r);
    void end    (void);
};


#endif
